package dev.peerat.framework;

import java.net.Socket;
import java.util.Arrays;

public class Client<U extends User> extends Thread{

	private HttpReader reader;
	private HttpWriter writer;
	private Router<U> router;
	private Locker<Context> logger;

	public Client(Socket socket, Router<U> router) throws Exception{
		this.reader = new HttpReader(socket);
		this.writer = new HttpWriter(socket);
		this.router = router;
		this.logger = router.getLogger();
	}

	@Override
	public void run(){
		Context context = null;
		try{
			String[] headers = reader.readLine().split("\\s");
			System.out.println(Arrays.toString(headers));
			reader.readHeaders();

			context = router.exec(RequestType.valueOf(headers[0]), headers[1], isLogin(reader), reader, writer);
			writer.flush();
			writer.close();
		}catch(Exception e){
			e.printStackTrace();
		}
		if(context != null) logger.setValue(context);
	}

	private User isLogin(HttpReader reader) throws Exception{
		String auth = reader.getHeader("Authorization");
		if(auth == null) return null;
		auth = auth.substring(7);
		try{
			return this.router.getUser(auth);
		}catch(Exception e){
			writer.response(401, "Access-Control-Allow-Origin: *");
			writer.flush();
			writer.close();
			throw e;
		}
	}
}